"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageSender = void 0;
const tsyringe_1 = require("tsyringe");
const constants_1 = require("../constants");
const TransportDecorator_1 = require("../decorators/transport/TransportDecorator");
const error_1 = require("../error");
const EnvelopeService_1 = require("./EnvelopeService");
const TransportService_1 = require("./TransportService");
const helpers_1 = require("./helpers");
let MessageSender = class MessageSender {
    constructor(envelopeService, transportService, messageRepository, logger) {
        this.envelopeService = envelopeService;
        this.transportService = transportService;
        this.messageRepository = messageRepository;
        this.logger = logger;
    }
    setOutboundTransporter(outboundTransporter) {
        this._outboundTransporter = outboundTransporter;
    }
    get outboundTransporter() {
        return this._outboundTransporter;
    }
    async packMessage(outboundMessage, keys) {
        const { connection, payload } = outboundMessage;
        const wireMessage = await this.envelopeService.packMessage(payload, keys);
        return { connection, payload: wireMessage };
    }
    async sendMessage(outboundMessage) {
        var e_1, _a;
        var _b;
        if (!this.outboundTransporter) {
            throw new error_1.AriesFrameworkError('Agent has no outbound transporter!');
        }
        const { connection } = outboundMessage;
        const { id, verkey, theirKey } = connection;
        this.logger.debug('Send outbound message', {
            connection: { id, verkey, theirKey },
            isUnpackedMessage: helpers_1.isUnpackedPackedMessage(outboundMessage),
            message: outboundMessage.payload,
            messageType: helpers_1.isUnpackedPackedMessage(outboundMessage) ? outboundMessage.payload.type : 'unknown',
        });
        const threadId = helpers_1.isUnpackedPackedMessage(outboundMessage) ? outboundMessage.payload.threadId : undefined;
        // Try sending over already open connection
        const session = this.transportService.findSessionByConnectionId(connection.id);
        if ((_b = session === null || session === void 0 ? void 0 : session.inboundMessage) === null || _b === void 0 ? void 0 : _b.hasReturnRouting(threadId)) {
            this.logger.debug(`Existing ${session.type} transport session has been found.`);
            try {
                let outboundPackage;
                // If outboundPackage is instance of AgentMessage we still need to pack
                if (helpers_1.isUnpackedPackedMessage(outboundMessage)) {
                    if (!session.keys) {
                        throw new error_1.AriesFrameworkError(`There are no keys for the given ${session.type} transport session.`);
                    }
                    outboundPackage = await this.packMessage(outboundMessage, session.keys);
                }
                // Otherwise we use the message that is already packed. This is often not the case
                // but happens with forwarding packed message
                else {
                    outboundPackage = outboundMessage;
                }
                await session.send(outboundPackage);
                return;
            }
            catch (error) {
                this.logger.info(`Sending an outbound message via session failed with error: ${error.message}.`, error);
            }
        }
        const services = this.transportService.findDidCommServices(connection);
        if (services.length === 0) {
            throw new error_1.AriesFrameworkError(`Connection with id ${connection.id} has no service!`);
        }
        this.logger.debug(`Found ${services.length} services for message to connection '${connection.id}' (${connection.theirLabel})`);
        try {
            for (var services_1 = __asyncValues(services), services_1_1; services_1_1 = await services_1.next(), !services_1_1.done;) {
                const service = services_1_1.value;
                // We can't send message to didcomm:transport/queue
                if (service.serviceEndpoint === constants_1.DID_COMM_TRANSPORT_QUEUE) {
                    this.logger.debug(`Skipping transport queue service for connection '${connection.id}'`, {
                        service,
                    });
                    continue;
                }
                this.logger.debug(`Sending outbound message to service:`, { connectionId: connection.id, service });
                try {
                    let outboundPackage;
                    // If outboundPackage is instance of AgentMessage we still need to pack
                    if (helpers_1.isUnpackedPackedMessage(outboundMessage)) {
                        const keys = {
                            recipientKeys: service.recipientKeys,
                            routingKeys: service.routingKeys || [],
                            senderKey: connection.verkey,
                        };
                        // Set return routing for message if we don't have an inbound endpoint for this connection
                        if (!this.transportService.hasInboundEndpoint(outboundMessage.connection.didDoc)) {
                            outboundMessage.payload.setReturnRouting(TransportDecorator_1.ReturnRouteTypes.all);
                        }
                        outboundPackage = await this.packMessage(outboundMessage, keys);
                        outboundPackage.responseRequested = outboundMessage.payload.hasReturnRouting();
                    }
                    else {
                        outboundPackage = outboundMessage;
                    }
                    outboundPackage.endpoint = service.serviceEndpoint;
                    await this.outboundTransporter.sendMessage(outboundPackage);
                    return;
                }
                catch (error) {
                    this.logger.debug(`Preparing outbound message to service with id ${service.id} failed with the following error:`, {
                        message: error.message,
                        error: error,
                    });
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (services_1_1 && !services_1_1.done && (_a = services_1.return)) await _a.call(services_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        // We didn't succeed to send the message over open session, or directly to serviceEndpoint
        // If the other party shared a queue service endpoint in their did doc we queue the message
        const queueService = services.find((s) => s.serviceEndpoint === constants_1.DID_COMM_TRANSPORT_QUEUE);
        if (queueService &&
            // FIXME: we can't currently add unpacked message to the queue. This is good for now
            // as forward messages are always packed. Allowing unpacked messages means
            // we can queue undeliverable messages
            !helpers_1.isUnpackedPackedMessage(outboundMessage)) {
            this.logger.debug(`Queue message for connection ${outboundMessage.connection.id} (${outboundMessage.connection.theirLabel})`);
            this.messageRepository.add(outboundMessage.connection.id, outboundMessage.payload);
        }
    }
};
MessageSender = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __param(2, tsyringe_1.inject(constants_1.InjectionSymbols.MessageRepository)),
    __param(3, tsyringe_1.inject(constants_1.InjectionSymbols.Logger)),
    __metadata("design:paramtypes", [EnvelopeService_1.EnvelopeService,
        TransportService_1.TransportService, Object, Object])
], MessageSender);
exports.MessageSender = MessageSender;
//# sourceMappingURL=MessageSender.js.map