"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IssueCredentialHandler = void 0;
const helpers_1 = require("../../../agent/helpers");
const messages_1 = require("../messages");
class IssueCredentialHandler {
    constructor(credentialService, agentConfig, credentialResponseCoordinator) {
        this.supportedMessages = [messages_1.IssueCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.credentialResponseCoordinator = credentialResponseCoordinator;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processCredential(messageContext);
        if (this.credentialResponseCoordinator.shouldAutoRespondToIssue(credentialRecord)) {
            return await this.createAck(credentialRecord, messageContext);
        }
    }
    async createAck(credentialRecord, messageContext) {
        this.agentConfig.logger.info(`Automatically sending acknowledgement with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        if (!messageContext.connection) {
            this.agentConfig.logger.error(`No connection on the messageContext`);
            return;
        }
        const { message } = await this.credentialService.createAck(credentialRecord);
        return helpers_1.createOutboundMessage(messageContext.connection, message);
    }
}
exports.IssueCredentialHandler = IssueCredentialHandler;
//# sourceMappingURL=IssueCredentialHandler.js.map