"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OfferCredentialHandler = void 0;
const helpers_1 = require("../../../agent/helpers");
const messages_1 = require("../messages");
class OfferCredentialHandler {
    constructor(credentialService, agentConfig, credentialResponseCoordinator) {
        this.supportedMessages = [messages_1.OfferCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.credentialReponseCoordinator = credentialResponseCoordinator;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processOffer(messageContext);
        if (this.credentialReponseCoordinator.shouldAutoRespondToOffer(credentialRecord)) {
            return await this.createRequest(credentialRecord, messageContext);
        }
    }
    async createRequest(credentialRecord, messageContext) {
        this.agentConfig.logger.info(`Automatically sending request with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        if (!messageContext.connection) {
            this.agentConfig.logger.error(`No connection on the messageContext`);
            return;
        }
        const { message } = await this.credentialService.createRequest(credentialRecord);
        return helpers_1.createOutboundMessage(messageContext.connection, message);
    }
}
exports.OfferCredentialHandler = OfferCredentialHandler;
//# sourceMappingURL=OfferCredentialHandler.js.map