"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestCredentialHandler = void 0;
const helpers_1 = require("../../../agent/helpers");
const messages_1 = require("../messages");
class RequestCredentialHandler {
    constructor(credentialService, agentConfig, credentialResponseCoordinator) {
        this.supportedMessages = [messages_1.RequestCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.credentialResponseCoordinator = credentialResponseCoordinator;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processRequest(messageContext);
        if (this.credentialResponseCoordinator.shouldAutoRespondToRequest(credentialRecord)) {
            return await this.createCredential(credentialRecord, messageContext);
        }
    }
    async createCredential(credentialRecord, messageContext) {
        this.agentConfig.logger.info(`Automatically sending credential with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        if (!messageContext.connection) {
            this.agentConfig.logger.error(`No connection on the messageContext`);
            return;
        }
        const { message } = await this.credentialService.createCredential(credentialRecord);
        return helpers_1.createOutboundMessage(messageContext.connection, message);
    }
}
exports.RequestCredentialHandler = RequestCredentialHandler;
//# sourceMappingURL=RequestCredentialHandler.js.map