"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PresentationHandler = void 0;
const helpers_1 = require("../../../agent/helpers");
const messages_1 = require("../messages");
class PresentationHandler {
    constructor(proofService, agentConfig, proofResponseCoordinator) {
        this.supportedMessages = [messages_1.PresentationMessage];
        this.proofService = proofService;
        this.agentConfig = agentConfig;
        this.proofResponseCoordinator = proofResponseCoordinator;
    }
    async handle(messageContext) {
        const proofRecord = await this.proofService.processPresentation(messageContext);
        if (this.proofResponseCoordinator.shouldAutoRespondToPresentation(proofRecord)) {
            return await this.createAck(proofRecord, messageContext);
        }
    }
    async createAck(proofRecord, messageContext) {
        this.agentConfig.logger.info(`Automatically sending acknowledgement with autoAccept on ${this.agentConfig.autoAcceptProofs}`);
        if (!messageContext.connection) {
            this.agentConfig.logger.error('No connection on the messageContext');
            return;
        }
        const { message } = await this.proofService.createAck(proofRecord);
        return helpers_1.createOutboundMessage(messageContext.connection, message);
    }
}
exports.PresentationHandler = PresentationHandler;
//# sourceMappingURL=PresentationHandler.js.map