"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndyWallet = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../agent/AgentConfig");
const error_1 = require("../error");
const JsonEncoder_1 = require("../utils/JsonEncoder");
const indyError_1 = require("../utils/indyError");
const error_2 = require("./error");
let IndyWallet = class IndyWallet {
    constructor(agentConfig) {
        this.logger = agentConfig.logger;
        this.indy = agentConfig.agentDependencies.indy;
    }
    get isInitialized() {
        return this.openWalletInfo !== undefined;
    }
    get publicDid() {
        return this.publicDidInfo;
    }
    get walletHandle() {
        if (!this.isInitialized || !this.openWalletInfo) {
            throw new error_1.AriesFrameworkError('Wallet has not been initialized yet');
        }
        return this.openWalletInfo.walletHandle;
    }
    get masterSecretId() {
        if (!this.isInitialized || !this.openWalletInfo) {
            throw new error_1.AriesFrameworkError('Wallet has not been initialized yet');
        }
        return this.openWalletInfo.masterSecretId;
    }
    async initialize(walletConfig, walletCredentials) {
        this.logger.info(`Initializing wallet '${walletConfig.id}'`, walletConfig);
        if (this.isInitialized) {
            throw new error_2.WalletError('Wallet instance already initialized. Close the currently opened wallet before re-initializing the wallet');
        }
        // Open wallet, creating if it doesn't exist yet
        try {
            await this.open(walletConfig, walletCredentials);
        }
        catch (error) {
            // If the wallet does not exist yet, create it and try to open again
            if (error instanceof error_2.WalletNotFoundError) {
                await this.create(walletConfig, walletCredentials);
                await this.open(walletConfig, walletCredentials);
            }
            else {
                throw error;
            }
        }
        this.logger.debug(`Wallet '${walletConfig.id}' initialized with handle '${this.walletHandle}'`);
    }
    /**
     * @throws {WalletDuplicateError} if the wallet already exists
     * @throws {WalletError} if another error occurs
     */
    async create(walletConfig, walletCredentials) {
        var _a;
        const storageType = (_a = walletConfig.storage_type) !== null && _a !== void 0 ? _a : 'SQLite';
        this.logger.debug(`Creating wallet '${walletConfig.id}' using ${storageType} storage`, {
            storageConfig: walletConfig.storage_config,
        });
        try {
            await this.indy.createWallet(walletConfig, walletCredentials);
        }
        catch (error) {
            if (indyError_1.isIndyError(error, 'WalletAlreadyExistsError')) {
                const errorMessage = `Wallet '${walletConfig.id}' already exists`;
                this.logger.debug(errorMessage);
                throw new error_2.WalletDuplicateError(errorMessage, {
                    walletType: 'IndyWallet',
                    cause: error,
                });
            }
            else {
                const errorMessage = `Error creating wallet '${walletConfig.id}'`;
                this.logger.error(errorMessage, {
                    error,
                    errorMessage: error.message,
                });
                throw new error_2.WalletError(errorMessage, { cause: error });
            }
        }
    }
    /**
     * @throws {WalletNotFoundError} if the wallet does not exist
     * @throws {WalletError} if another error occurs
     */
    async open(walletConfig, walletCredentials) {
        if (this.isInitialized) {
            throw new error_2.WalletError('Wallet instance already initialized. Close the currently opened wallet before re-initializing the wallet');
        }
        try {
            const walletHandle = await this.indy.openWallet(walletConfig, walletCredentials);
            const masterSecretId = await this.createMasterSecret(walletHandle, walletConfig.id);
            this.openWalletInfo = {
                walletConfig,
                walletCredentials,
                walletHandle,
                masterSecretId,
            };
        }
        catch (error) {
            if (indyError_1.isIndyError(error, 'WalletNotFoundError')) {
                const errorMessage = `Wallet '${walletConfig.id}' not found`;
                this.logger.debug(errorMessage);
                throw new error_2.WalletNotFoundError(errorMessage, {
                    walletType: 'IndyWallet',
                    cause: error,
                });
            }
            else {
                const errorMessage = `Error opening wallet '${walletConfig.id}'`;
                this.logger.error(errorMessage, {
                    error,
                    errorMessage: error.message,
                });
                throw new error_2.WalletError(errorMessage, { cause: error });
            }
        }
    }
    /**
     * @throws {WalletNotFoundError} if the wallet does not exist
     * @throws {WalletError} if another error occurs
     */
    async delete() {
        const walletInfo = this.openWalletInfo;
        if (!this.isInitialized || !walletInfo) {
            throw new error_2.WalletError('Can not delete wallet that is not initialized. Make sure to call initialize before deleting the wallet');
        }
        this.logger.info(`Deleting wallet '${walletInfo.walletConfig.id}'`);
        await this.close();
        try {
            await this.indy.deleteWallet(walletInfo.walletConfig, walletInfo.walletCredentials);
        }
        catch (error) {
            if (indyError_1.isIndyError(error, 'WalletNotFoundError')) {
                const errorMessage = `Error deleting wallet: wallet '${walletInfo.walletConfig.id}' not found`;
                this.logger.debug(errorMessage);
                throw new error_2.WalletNotFoundError(errorMessage, {
                    walletType: 'IndyWallet',
                    cause: error,
                });
            }
            else {
                const errorMessage = `Error deleting wallet '${walletInfo.walletConfig.id}': ${error.message}`;
                this.logger.error(errorMessage, {
                    error,
                    errorMessage: error.message,
                });
                throw new error_2.WalletError(errorMessage, { cause: error });
            }
        }
    }
    /**
     * @throws {WalletError} if the wallet is already closed or another error occurs
     */
    async close() {
        try {
            await this.indy.closeWallet(this.walletHandle);
            this.openWalletInfo = undefined;
            this.publicDidInfo = undefined;
        }
        catch (error) {
            if (indyError_1.isIndyError(error, 'WalletInvalidHandle')) {
                const errorMessage = `Error closing wallet: wallet already closed`;
                this.logger.debug(errorMessage);
                throw new error_2.WalletError(errorMessage, {
                    cause: error,
                });
            }
            else {
                const errorMessage = `Error closing wallet': ${error.message}`;
                this.logger.error(errorMessage, {
                    error,
                    errorMessage: error.message,
                });
                throw new error_2.WalletError(errorMessage, { cause: error });
            }
        }
    }
    /**
     * Create master secret with specified id in currently opened wallet.
     *
     * If a master secret by this id already exists in the current wallet, the method
     * will return without doing anything.
     *
     * @throws {WalletError} if an error occurs
     */
    async createMasterSecret(walletHandle, masterSecretId) {
        this.logger.debug(`Creating master secret with id '${masterSecretId}' in wallet with handle '${walletHandle}'`);
        try {
            await this.indy.proverCreateMasterSecret(walletHandle, masterSecretId);
            return masterSecretId;
        }
        catch (error) {
            if (indyError_1.isIndyError(error, 'AnoncredsMasterSecretDuplicateNameError')) {
                // master secret id is the same as the master secret id passed in the create function
                // so if it already exists we can just assign it.
                this.logger.debug(`Master secret with id '${masterSecretId}' already exists in wallet with handle '${walletHandle}'`, {
                    indyError: 'AnoncredsMasterSecretDuplicateNameError',
                });
                return masterSecretId;
            }
            else {
                this.logger.error(`Error creating master secret with id ${masterSecretId}`, {
                    indyError: error.indyName,
                    error,
                });
                throw new error_2.WalletError(`Error creating master secret with id ${masterSecretId} in wallet with handle '${walletHandle}'`, { cause: error });
            }
        }
    }
    async initPublicDid(didConfig) {
        const [did, verkey] = await this.createDid(didConfig);
        this.publicDidInfo = {
            did,
            verkey,
        };
    }
    async createDid(didConfig) {
        return this.indy.createAndStoreMyDid(this.walletHandle, didConfig || {});
    }
    async pack(payload, recipientKeys, senderVk) {
        const messageRaw = JsonEncoder_1.JsonEncoder.toBuffer(payload);
        const packedMessage = await this.indy.packMessage(this.walletHandle, messageRaw, recipientKeys, senderVk);
        return JsonEncoder_1.JsonEncoder.fromBuffer(packedMessage);
    }
    async unpack(messagePackage) {
        const unpackedMessageBuffer = await this.indy.unpackMessage(this.walletHandle, JsonEncoder_1.JsonEncoder.toBuffer(messagePackage));
        const unpackedMessage = JsonEncoder_1.JsonEncoder.fromBuffer(unpackedMessageBuffer);
        return Object.assign(Object.assign({}, unpackedMessage), { message: JsonEncoder_1.JsonEncoder.fromString(unpackedMessage.message) });
    }
    async sign(data, verkey) {
        const signatureBuffer = await this.indy.cryptoSign(this.walletHandle, verkey, data);
        return signatureBuffer;
    }
    async verify(signerVerkey, data, signature) {
        // check signature
        const isValid = await this.indy.cryptoVerify(signerVerkey, data, signature);
        return isValid;
    }
    async addWalletRecord(type, id, value, tags) {
        return this.indy.addWalletRecord(this.walletHandle, type, id, value, tags);
    }
    async updateWalletRecordValue(type, id, value) {
        return this.indy.updateWalletRecordValue(this.walletHandle, type, id, value);
    }
    async updateWalletRecordTags(type, id, tags) {
        return this.indy.addWalletRecordTags(this.walletHandle, type, id, tags);
    }
    async deleteWalletRecord(type, id) {
        return this.indy.deleteWalletRecord(this.walletHandle, type, id);
    }
    async search(type, query, options) {
        const sh = await this.indy.openWalletSearch(this.walletHandle, type, query, options);
        const generator = function (indy, wh) {
            return __asyncGenerator(this, arguments, function* () {
                try {
                    while (true) {
                        // count should probably be exported as a config?
                        const recordSearch = yield __await(indy.fetchWalletSearchNextRecords(wh, sh, 10));
                        for (const record of recordSearch.records) {
                            yield yield __await(record);
                        }
                    }
                }
                catch (error) {
                    // pass
                }
                finally {
                    yield __await(indy.closeWalletSearch(sh));
                }
            });
        };
        return generator(this.indy, this.walletHandle);
    }
    getWalletRecord(type, id, options) {
        return this.indy.getWalletRecord(this.walletHandle, type, id, options);
    }
    signRequest(myDid, request) {
        return this.indy.signRequest(this.walletHandle, myDid, request);
    }
    async generateNonce() {
        return this.indy.generateNonce();
    }
};
IndyWallet = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [AgentConfig_1.AgentConfig])
], IndyWallet);
exports.IndyWallet = IndyWallet;
//# sourceMappingURL=IndyWallet.js.map